import java.util.*;
import java.awt.*;

import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;

import java.net.URL;
import java.io.IOException;
import java.io.File;
public class TreeDemo extends JPanel {
    private static JEditorPane htmlPane;

    public TreeDemo(String filename) {
        super(new GridLayout(1,0));

		DefaultMutableTreeNode top = createFileNodes(new File(filename));

		JScrollPane treeView = new JScrollPane(new ZoomTree(top));

        htmlPane = new JEditorPane();
        JScrollPane htmlView = new JScrollPane(htmlPane);

        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setLeftComponent(treeView);
        splitPane.setRightComponent(htmlView);

        splitPane.setPreferredSize(new Dimension(1000, 700));
		splitPane.setDividerLocation(700);

        add(splitPane);
    }


	
	
    public static void displayURL(URL url) {
        try {
            if (url != null) {
                htmlPane.setPage(url);
            }
        } catch (IOException e) {
            System.err.println("Attempted to read a bad URL: " + url);
        }
    }

    public static void main(String[] args) {
             try {
                UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
            } catch (Exception e) {
                System.err.println("Couldn't use system look and feel.");
            }
 
		String filename = ".";
		if (args.length > 0)
			filename = args[0];
		
        JFrame.setDefaultLookAndFeelDecorated(true);

        JFrame frame = new JFrame("TreeDemo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        TreeDemo newContentPane = new TreeDemo(filename);
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);

        frame.pack();
        frame.setVisible(true);
    }

	
	public DefaultMutableTreeNode createFileNodes(File file){
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(new FileNode(file));
		
        if (file.isDirectory()) {
            String[] tabFichiers = file.list();
            for (int i = 0; i < tabFichiers.length; i++) {
                File fichier = new File(file.getPath() +
                                        File.separator + tabFichiers[i]);
                node.add(createFileNodes(fichier));
            }
        }
		
		return node;
	}
	
}
    class FileNode {
		public File file;
        public URL url;

        public FileNode(File _file) {
			file = _file;
			try{
			url = file.toURL(); 
			}
			catch(Exception e){System.out.println(""+e);}
        }

        public String toString() {
            return file.getName();
        }
    }
