
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.File;
import java.net.URL;

public class ZoomTree extends JComponent
{
	DefaultMutableTreeNode root;
	ZoomTree(DefaultMutableTreeNode root){
		this.root = root;
		
		MouseDispatcher mouseDispatcher = new MouseDispatcher();
		addMouseListener(mouseDispatcher);
		addMouseMotionListener(mouseDispatcher);
		
	}
	
	Vector visible_objects = new Vector();
	
	public void paint(Graphics g){
		super.paint(g);
		g.setColor(Color.black);
		
		visible_objects = new Vector();
		g.setFont(new Font("SansSerif", Font.PLAIN, Math.max(12, (int)(12*scale))));
		paint_node(g, root, center.x, center.y, 0);
		
		g.setFont(new Font("SansSerif", Font.PLAIN, 12));
		g.setColor(Color.red);
		g.drawString("}EX{^hbO(㉺) = Scroll", 480, 20);
		g.drawString("}EXE{^hbO(E) = Zoom", 480, 40);
	}
	
	public double paint_node(Graphics g, DefaultMutableTreeNode node, double x, double y, int level){
		String string = node.toString();
		Rectangle bbox = new Rectangle((int)x, (int)y,
									   (int)(g.getFontMetrics().stringWidth(string)),
									   (int)(g.getFontMetrics().getHeight()));
		if (Math.pow(2, level) < 2+2*32*scale)
		if (bbox.x < this.getWidth() && bbox.x+bbox.width > 0 &&
			bbox.y < this.getHeight() && bbox.y+bbox.height > 0){
			
			visible_objects.addElement(new VisibleObject(bbox, ((FileNode)node.getUserObject()).url));
			//g.drawRect(bbox.x, bbox.y, bbox.width, bbox.height);
			g.drawString(string, (int)x, (int)(y+g.getFontMetrics().getAscent()));
		}
		
		
		
		double dy = 14 * scale;
		for(Enumeration e = node.children(); e.hasMoreElements(); ){
				DefaultMutableTreeNode child = (DefaultMutableTreeNode) e.nextElement();
				dy += paint_node(g, child, x+40, y+dy, level+1);
		}
		
		return dy;
	}
	class VisibleObject {
		Rectangle bbox;
		URL url;
		VisibleObject(Rectangle _bbox, URL _url){
			bbox = _bbox;
			url = _url;
		}
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	//
	// zoom
	//
	Point start_p;
	Point prev_p;
	Point center = new Point(0,0);
	double scale = 1;
	public void start_zooming(Point p){
		prev_p = p;
		start_p = p;
	}
	public void zooming(Point p){
		double prev_scale = scale;
		scale *= Math.pow(1.02, (p.x - prev_p.x));
		
		//center.x -= (start_p.x -center.x)*(scale / prev_scale -1);
		center.y -= (start_p.y -center.y)*(scale / prev_scale -1);
				   
		prev_p = p;
		repaint();
	}
	
	
	
	
	//
	// scroll
	//
	public void start_scrolling(Point p){
		prev_p = p;
	}
	public void scrolling(Point p){
		//center.x += (int)((p.x-prev_p.x));
		center.y += (int)((p.y-prev_p.y));
		prev_p = p;
		repaint();
	}	
	
	
	
	//
	public void clicked(Point p){
		for(int i=0; i<visible_objects.size(); i++){
			VisibleObject object = (VisibleObject) visible_objects.elementAt(i);
			if (object.bbox.contains(p))
				TreeDemo.displayURL(object.url);
		}
	}
	
	
	

   
	//   
	// central event dispatcher
	//
	public int operation_status;
	public static final int NONE = 0;
	public static final int ZOOMING = 1;
	public static final int SCROLLING = 2;
	
	public class MouseDispatcher 
		extends MouseAdapter implements MouseMotionListener{

  		public void mouseMoved(MouseEvent e) {
  		}
  		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			if ((e.getModifiers() & e.BUTTON3_MASK) != 0){
				operation_status = ZOOMING;
				start_zooming(p);
			}
			else {
				operation_status = SCROLLING;
				start_scrolling(p);
			}
		}
  		public void mouseDragged(MouseEvent e) {
			Point p = e.getPoint();
			switch(operation_status){
			case ZOOMING:
				zooming(p);
				break;
			case SCROLLING:
				scrolling(p);
				break;
			}
		}	
		public void mouseReleased(MouseEvent e) {
			operation_status = NONE;
			repaint();
		}
		
		public void mouseClicked(MouseEvent e){
			clicked(e.getPoint());	
		}
    }	
}
